@@ -43,6 +43,7 @@ class Estimator(Term):
     warnings.simplefilter("ignore", FutureWarning)
     warnings.simplefilter("ignore", SyntaxWarning)
 
+    @deprecate("Estimator", new="ODEHandler", version="2.19.1-zeta-fu1")
     def __init__(self, record_df, model, population, tau=None, **kwargs):
         # ODE model
         self.model = self._ensure_subclass(model, ModelBase, name="model")
@@ -52,8 +53,8 @@ class Estimator(Term):
         # Dataset
         if not set(self.NLOC_COLUMNS).issubset(record_df.columns):
             record_df = model.restore(record_df)
-        self.record_df = self._ensure_dataframe(
-            record_df, name="record_df", columns=self.NLOC_COLUMNS)
+        self._ensure_dataframe(record_df, name="record_df", columns=self.NLOC_COLUMNS)
+        self.record_df = record_df.copy()
         # Settings for simulation
         self.population = self._ensure_population(population)
         df = model.tau_free(self.record_df, population, tau=None)
